--[[
Copyright 2010, 2011 João Cardoso
Cornucopia is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Cornucopia.

Cornucopia is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cornucopia is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cornucopia. If not, see <http://www.gnu.org/licenses/>.
--]]

local Cornucopia = CreateFrame('Frame', 'Cornucopia')
BINDING_HEADER_CORNUCOPIA = 'Cornucopia'
Cornucopia_Locals = {}


--[[ Startup ]]--

function Cornucopia:Startup()
	self:SetScript('OnEvent', self.Initialize)
	self:RegisterEvent('VARIABLES_LOADED')
	self.Groups, self.Bars = {}, {}
	self.__index = self
end

function Cornucopia:Initialize()
	-- Broker
	for id, data in LibStub('LibDataBroker-1.1'):DataObjectIterator() do
		if data.type == 'Movable Bar' then
			self:CreateBar(id, data)
		elseif data.type == 'Bar Group' then
			self:CreateGroup(id, data)
		end
	end

	-- Initialize
	for id, data in pairs(self.Groups) do
		self:InitializeGroup(id, data)
	end
	
	for id, bar in pairs(self.Bars) do
		self:InitializeBar(bar)
	end
	
	self.CreateGroup, self.InitializeGroup = nil
	self.Startup, self.Initialize = nil
	
	-- Settings
	if Cornucopia_HideButton then
		CornucopiaButton:Hide()
	end
	
	if not Cornucopia_Tutorials then
		LoadAddOn('Cornucopia_Config')
	end
end

function Cornucopia:InitializeSets(data)
	local sets = data.sets or {}
	local value = _G[sets] or type(sets) == 'table' and sets or data.defaults
	
	data.defaults = nil
	data.sets = value
	
	_G[sets] = value
	return value
end


--[[ Bars ]]--

function Cornucopia:CreateBar(id, data)
	local bar = setmetatable(CreateFrame('Frame', nil, UIParent, data.templates), self)
	bar.data, data.frame = data, bar
	
	self.Bars[id] = bar
	return data
end

function Cornucopia:InitializeBar(bar)
	local data = bar.data
	local sets = self:InitializeSets(data)
	bar:SetPoint(sets.point, self.Bars[sets.anchor] or UIParent, sets.relPoint or sets.point, sets.x or 0, sets.y or 0)
	bar:SetSize(sets.width or 1, sets.height or 1)
	bar:SetScale(sets.scale or 1)
	bar:SetAlpha(sets.alpha or 1)
	bar:SetClampedToScreen(true)
	bar.sets = sets
	
	if sets.hide then
		bar:Hide()
	end

	if data.OnInitialize then
		data:OnInitialize()
		data.OnInitialize = nil
	end
end


--[[ Groups ]]--

function Cornucopia:CreateGroup(id, data)
	self.Groups[id] = data
	data.__index = data
	return data
end

function Cornucopia:InitializeGroup(id, data)
	local sets = self:InitializeSets(data)
	data.group = data
	
	for i, set in pairs(sets) do
		local new = setmetatable({}, data)
		new.name = data.name .. ' #' .. i
		new.sets = set
		new.index = i
		
		self:CreateBar(id..i, new)
	end
end

Cornucopia:Startup()