--[[
Copyright 2008, 2009, 2010, 2011 João Cardoso
Sushi is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Sushi.

Sushi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Sushi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Sushi. If not, see <http://www.gnu.org/licenses/>.
--]]

local Sushi = LibStub('Sushi-2.0')
local EditBox, old = Sushi:NewWidgetClass('EditBox', {
	frameType = 'EditBox',
	template = 'InputBoxTemplate',
	insets = {top = 10, bottom = -4, left = 20, right = 20},
	version = 2,
})

if not EditBox then
	return
end


--[[ Events ]]--

function EditBox:OnCreate()
	local Label = self:CreateFontString()
	Label:SetPoint('BOTTOMLEFT', self, 'TOPLEFT', -7, -4)
	Label:SetJustifyH('Left')
	
	self.SetValue = self.SetText
	self.GetValue = self.GetText
	self.Label = Label
	
	self:SetScript('OnEditFocusGained', self.OnEditFocusGained)
	self:SetScript('OnEscapePressed', self.OnEscapePressed)
	self:SetScript('OnEnterPressed', self.OnEnterPressed)
	self:SetAutoFocus(false)
	self:SetHeight(35)
end

function EditBox:OnAcquire()
	self:SetWidth(150)
	self:UpdateFonts()
end

function EditBox:OnRelease()
	self.disabled, self.small, self.desc = nil
	self:SetPassword(nil)
	self:SetNumeric(nil)
	self:SetLabel(nil)
	self:SetValue('')
	self:ClearFocus()
end


--[[ Scripts ]]--

function EditBox:OnEditFocusGained()
	self.value = self:GetValue()
end

function EditBox:OnEnterPressed()
	self:FireCallback('OnValueChanged', self:GetValue())
	self:FireCallback('OnUpdate')
	self:ClearFocus()
end

function EditBox:OnEscapePressed()
	self:SetValue(self.value or '')
	self:ClearFocus()
end


--[[ Label ]]--

function EditBox:SetLabel(label)
	self.Label:SetText(label)
end

function EditBox:GetLabel()
	return self.Label:GetText()
end


--[[ Font ]]--

function EditBox:SetDisabled(disabled)
	if disabled then
		self:ClearFocus()
	end
	
	self.disabled = disabled
	self:EnableMouse(not disabled)
	self:UpdateFonts()
end

function EditBox:IsDisabled()
	return self.disabled
end

function EditBox:SetSmall(small)
	self.small = small
	self:UpdateFonts()
end

function EditBox:IsSmall()
	return self.small
end

function EditBox:UpdateFonts()
	local font = 'GameFont'
	if not self:IsDisabled() then
		font = font .. '%s'
	else
		font = font .. 'Disable'
	end
	
	self.Label:SetFontObject(font:format('Normal') .. (self:IsSmall() and 'Small' or ''))
	self:SetFontObject(font:format('Highlight') .. 'Small')
end


--[[ Embedding ]]--

if not old then
	Sushi:GetClass('Dropdown'):SpecificEmbed(EditBox, 'SetDescription', 'GetDescription', 'OnEnter', 'OnLeave')
else
	EditBox:UpdateEmbeds()
end