--[[
Copyright 2008, 2009, 2010, 2011 João Cardoso
Sushi is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Sushi.

Sushi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Sushi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Sushi. If not, see <http://www.gnu.org/licenses/>.
--]]

local Sushi = LibStub('Sushi-2.0')
local Container = Sushi:GetClass('Container')
local TabGroup, old = Sushi:NewWidgetClass('TabGroup', {version = 1})
if not TabGroup then
	return
end


--[[ Startup ]]--

function TabGroup:OnCreate()
	Container.OnCreate(self)
	
	self.tabs = {}
	self.tabOrder = {}
	self:SetHeight(40)
	self:SetLayout('Row')
	self:SetChildren('UpdateTabs')
end

function TabGroup:UpdateHeight() end -- fake function

function TabGroup:OnAcquire()
	self:SetStyle('Tooltip')
	self:SetBackdrop(nil)
	self:SetWidth(nil)
end

function TabGroup:OnRelease()
	self:SelectTab(nil)
	self:ClearTabs()
end


--[[ Tabs ]]--

function TabGroup:AddTab(name, text, desc, disabled)
	if not self.tabs[name] then
		tinsert(self.tabOrder, name)
	end
	
	self.tabs[name] = {text or name, desc, disabled}
	self:UpdateChildren()
end

function TabGroup:GetTab(name)
	return name and self.tabs[name] and unpack(self.tabs[name])
end

function TabGroup:UpdateTabs()
	local selection = self:GetSelectedTab()
	for i, name in pairs(self.tabOrder) do
		local tab = self:CreateChild(self.style..'Tab')
		local data = self.tabs[name]
		
		tab:RegisterCallback('OnClick', function() self:SelectTab(name) end)
		tab:SetSelected(name == selection)
		tab:SetDescription(data[2])
		tab:SetDisabled(data[3])
		tab:SetText(data[1])
	end
end

function TabGroup:ClearTabs()
	wipe(self.tabs)
	wipe(self.tabOrder)
	self:UpdateChildren()
end


--[[ Selection ]]--

function TabGroup:SelectTab(selection)
	self.selectedTab = selection
	self:UpdateChildren()
	
	self:FireCallback('OnTabSelected', selection)
	self:FireCallback('OnSelection', selection)
	self:FireCallback('OnUpdate')
end

function TabGroup:GetSelectedTab()
	return self.selectedTab
end


--[[ Style ]]--

function TabGroup:SetStyle(style)
	self.style = style
end

function TabGroup:GetStyle()
	return self.style
end


--[[ Embedding ]]--

if not old then
	Container:Embed(TabGroup, 'OnCreate', 'OnAcquire', 'OnRelease', 'SetHeight', 'UpdateHeight', 'frameType', 'template')
else
	TabGroup:UpdateEmbeds()
end