--[[
Copyright 2008, 2009, 2010, 2011 João Cardoso
Sushi is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Sushi.

Sushi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Sushi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Sushi. If not, see <http://www.gnu.org/licenses/>.
--]]

local Sushi = LibStub('Sushi-2.0')
local Tab = Sushi:NewWidgetClass('TabTemplate', {
	insets = {left = -7, right = -7},
	version = 1
})

if not Tab then
	return
end


--[[ Events ]]--

function Tab:OnRelease()
	self:SetDisabled(nil)
	self:SetSelected(nil)
	self:SetText(nil)
end

function Tab:OnClick()
	PlaySound("igCharacterInfoTab")
	self:FireCallback('OnClick')
end


--[[ Select ]]--

function Tab:SetSelected(selected)
	self.selected = selected
	self:UpdateState()
end

function Tab:IsSelected()
	return self.selected
end


--[[ Disable ]]--

function Tab:SetDisabled(disabled)
	self.disabled = disabled
	self:UpdateState()
end

function Tab:IsDisabled()
	return self.disabled
end


--[[ Other API ]]--

function Tab:SetText(text)
	 self:GetFontString():SetText(text)
	 PanelTemplates_TabResize(self, 0)
end

function Tab:UpdateState(selected)
	if self.disabled then
		PanelTemplates_SetDisabledTabState(self)
	elseif self.selected then
		PanelTemplates_SelectTab(self)
	else
		PanelTemplates_DeselectTab(self)
	end
end


--[[ Embedding ]]--

if not old then
	Sushi:GetClass('Button'):Embed(Tab, 'OnRelease', 'OnClick', 'SetDisabled', 'SetText', 'SetSmall', 'IsSmall', 'frameType')
else
	Tab:UpdateEmbeds()
end


--[[ Styles ]]--

Tab:Embed(Sushi:NewWidgetClass('TooltipTab', {template = 'OptionsFrameTabButtonTemplate'}), 'frameType', 'insets')
Tab:Embed(Sushi:NewWidgetClass('PanelTab', {template = 'CharacterFrameTabButtonTemplate'}), 'frameType', 'insets')
Tab:Embed(Sushi:NewWidgetClass('DialogTab', {template = 'TabButtonTemplate'}), 'frameType', 'insets')