--[[
Copyright 2008, 2009, 2010, 2011 João Cardoso
Sushi is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Sushi.

Sushi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Sushi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Sushi. If not, see <http://www.gnu.org/licenses/>.
--]]

local Sushi = LibStub('Sushi-2.0')
local Button = Sushi:NewWidgetClass('TextureButton', {
	insets = {top = 10, bottom = 10, left = 10, right = 10},
	frameType = 'Button',
	version = 1,
})

if not Button then
	return
end


--[[ Events ]]--

function Button:OnCreate()
	self:SetScript('OnClick', self.OnClick)
	self:SetScript('OnEnter', self.OnEnter)
	self:SetScript('OnLeave', self.OnLeave)
	
	local Normal = self:CreateTexture()
	Normal:SetNonBlocking(true)
	Normal:SetAllPoints()
	self:SetNormalTexture(Normal)
	
	local Pushed = self:CreateTexture()
	Pushed:SetPoint('BOTTOMRIGHT', 2, -2)
	Pushed:SetPoint('TOPLEFT', 2, -2)
	Pushed:SetNonBlocking(true)
	self:SetPushedTexture(Pushed)
	
	local Highlight = self:CreateTexture()
	Highlight:SetTexture('Interface\\BUTTONS\\UI-Common-MouseHilight')
	Highlight:SetTexCoord(0.15, 0.85, 0.15, 0.85)
	Highlight:SetBlendMode('ADD')
	self:SetHighlightTexture(Highlight)
end

function Button:OnRelease()
	self:SetDescription(nil)
	self:SetTexture(nil)
end

function Button:OnClick()
	PlaySound('igMainMenuOptionCheckBoxOn')
	self:FireCallback('OnClick')
	self:FireCallback('OnUpdate')
end


--[[ API ]]--

function Button:SetTexture(image)
	local kind = type(image)
	local texture = kind == 'string' and image
	local Highlight = self:GetHighlightTexture()
	
	if texture then
		Highlight:SetPoint('BOTTOMRIGHT', 10, -10)
		Highlight:SetPoint('TOPLEFT', -10, 10)
	else
		Highlight:SetPoint('BOTTOMRIGHT')
		Highlight:SetPoint('TOPLEFT')
	end
	
	self:SetBackdrop(not texture and image or nil)
	self:GetNormalTexture():SetTexture(texture)
	self:GetPushedTexture():SetTexture(texture)
end

function Button:GetTexture()
	return self:GetNormalTexture():GetTexture() or self:GetBackdrop() or nil
end


--[[ Tooltip ]]--

function Button:SetDescription(desc)
	self.desc = desc
end

function Button:GetDescription()
	return self.desc
end

function Button:OnEnter()
	local desc = self:GetDescription()
	if desc then
		GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
		GameTooltip:SetText(desc, nil, nil, nil)
		GameTooltip:Show()
	end
end

function Button:OnLeave()
	if self:GetDescription() then
		GameTooltip:Hide()
	end
end


--[[ Embedding ]]--

Button:UpdateEmbeds()