--[[
Copyright 2008, 2009, 2010, 2011 João Cardoso
Sushi is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Sushi.

Sushi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Sushi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Sushi. If not, see <http://www.gnu.org/licenses/>.
--]]

local Sushi = LibStub('Sushi-2.0')
local Dropdown = Sushi:GetClass('Dropdown')
local TextureDrop, old = Sushi:NewWidgetClass('TextureDropdown', {version = 1})
if not TextureDrop then
	return
end

local DropList = TextureDrop.List or Sushi:CreateWidget('Container')
DropList:SetParent(UIParent) DropList:SetFrameStrata('FULLSCREEN_DIALOG')
DropList:SetLayout('Rectangular', {maxWidth = 270})
DropList:SetToplevel(1)
DropList:SetHeight(nil)
DropList:SetWidth(nil)
DropList:Hide()

DropList:SetChildren(function()
	local self = DropList.parent
	local height = self:GetHeight() - 7
	local width = self:GetWidth() - 57
	
	for value, image, desc in self:IterateLines() do
		local button = DropList:CreateChild('TextureButton')
		button:SetHeight(height) button:SetWidth(width)
		button:SetDescription(desc)
		button:SetTexture(image)
		
		button:RegisterCallback('OnClick', function()
			self:FireCallback('OnValueChanged', value)
			self:FireCallback('OnLineSelected', value)
			self:FireCallback('OnUpdate')
		end)
	end
end)

DropList:SetBackdrop({
	bgFile = 'Interface\\DialogFrame\\UI-DialogBox-Background-Dark',
	edgeFile = 'Interface\\DialogFrame\\UI-DialogBox-Border',
	insets = {left = 11, right = 11, top = 11, bottom = 9},
	edgeSize = 32, tileSize = 32, tile = true,
	edgeColor = {1,1,1,1},
})


--[[ Startup ]]--

function TextureDrop:OnCreate()
	local name = self:GetName()
	Dropdown.OnCreate(self)
	
	-- Left
	local TopLeft = _G[name .. 'Left']
	TopLeft:SetTexCoord(0, 0.1953125, 0, 0.4)
	TopLeft:SetPoint('TOPLEFT', 0, 13)
	TopLeft:SetHeight(19.2)
	
	local BotLeft = self:CreateTexture()
	BotLeft:SetTexture('Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame')
	BotLeft:SetHeight(15) BotLeft:SetWidth(25)
	BotLeft:SetTexCoord(0, 0.1953125, 0.5, 1)
	BotLeft:SetPoint('BOTTOMLEFT')
	
	local Left = self:CreateTexture()
	Left:SetTexture('Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame')
	Left:SetTexCoord(0, 0.1953125, 0.5, 0.5)
	Left:SetPoint('TOP', TopLeft, 'BOTTOM')
	Left:SetPoint('BOTTOM', BotLeft, 'TOP')
	Left:SetWidth(25)
	
	-- Right
	local TopRight = _G[name .. 'Right']
	TopRight:SetTexCoord(0.8046875, 1, 0, 0.4)
	TopRight:SetPoint('TOPRIGHT', 0, 13)
	TopRight:SetHeight(19.2)
	
	local BotRight = self:CreateTexture()
	BotRight:SetTexture('Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame')
	BotRight:SetHeight(15) BotRight:SetWidth(25)
	BotRight:SetTexCoord(0.8046875, 1, 0.5, 1)
	BotRight:SetPoint('BOTTOMRIGHT')
	
	local Right = self:CreateTexture()
	Right:SetTexture('Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame')
	Right:SetTexCoord(0.8046875, 1, 0.5, 0.5)
	Right:SetPoint('TOP', TopRight, 'BOTTOM')
	Right:SetPoint('BOTTOM', BotRight, 'TOP')
	Right:SetWidth(25)
	
	-- Middle
	local TopMiddle = _G[name .. 'Middle']
	TopMiddle:SetTexCoord(0.1953125, 0.8046875, 0, 0.4)
	TopMiddle:SetHeight(19.2)
	
	local BotMiddle = self:CreateTexture()
	BotMiddle:SetTexture('Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame')
	BotMiddle:SetTexCoord(0.1953125, 0.8046875, 0.5, 1)
	BotMiddle:SetPoint('RIGHT', BotRight, 'LEFT')
	BotMiddle:SetPoint('LEFT', BotLeft, 'RIGHT')
	BotMiddle:SetHeight(15)
	
	local Middle = self:CreateTexture()
	Middle:SetTexture('Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame')
	Middle:SetTexCoord(0.1953125, 0.8046875, 0.5, 0.5)
	Middle:SetPoint('TOP', TopMiddle, 'BOTTOM')
	Middle:SetPoint('BOTTOM', BotMiddle, 'TOP')
	Middle:SetPoint('RIGHT', Right, 'LEFT')
	Middle:SetPoint('LEFT', Left, 'RIGHT')
	
	-- Ohter
	self.Button:ClearAllPoints()
	self.Button:SetPoint('TOPRIGHT', -16, 0)
	self.Button:SetScript('OnClick', function() self:OnClick() end)
	
	self.Display = Sushi:CreateWidget('TextureButton', self)
	self.Display:SetPoint('BOTTOMRIGHT', Middle, -15, 5)
	self.Display:SetPoint('TOPLEFT', Middle, 15, -5)
	self.Display:EnableMouse(false)
end

function TextureDrop:OnAcquire()
	self:SetWidth(120)
	self:SetHeight(70)
	self:UpdateFonts()
end

function TextureDrop:OnClick()
	PlaySound('igMainMenuOptionCheckBoxOn')
	DropList:ClearAllPoints()

	if not DropList:IsShown() or DropList.parent ~= self then
		CloseDropDownMenus()
		
		DropList.parent = self
		DropList:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 16, -4)
		DropList:Show()
	else
		DropList:Hide()
	end
	
end


--[[ API ]]--

function TextureDrop:SetValue(value)
	self.Display:SetTexture(self.linesNames[value])
	self.selectedValue = value
end


--[[ Embedding ]]--

if not old then
	Dropdown:Embed(TextureDrop, 'OnCreate', 'OnAcquire', 'SetValue', 'Initialize', 'frameType', 'template', 'insets')
	hooksecurefunc('ToggleDropDownMenu', function() DropList:Hide() end)
	hooksecurefunc('CloseDropDownMenus', function() DropList:Hide() end)
	TextureDrop.List = DropList
else
	TextureDrop:UpdateEmbeds()
end