--[[
Copyright 2010, 2011 João Cardoso
Cornucopia is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Cornucopia.

Cornucopia is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cornucopia is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cornucopia. If not, see <http://www.gnu.org/licenses/>.
--]]

function CornucopiaActions_StartBinder()
	local binder = CreateFrame('Frame', 'CornucopiaActionsBinder', nil, 'GlowBoxTemplate')
	local bar, parent, key
	local overlays = {}
	
	local arrow = CreateFrame('Frame', '$parentArrow', binder, 'GlowBoxArrowTemplate')
	arrow:SetPoint('TOP', binder, 'BOTTOM', 4, 0)
	
	local text = binder:CreateFontString(nil, nil, 'GameFontHighlightLeft')
	text:SetText('Mouse over a button and press a key to set a binging.|nPressing "Esc" will remove the binding.')
	text:SetPoint('TOPLEFT', 16, -14)
	text:SetWidth(185)
	
	local close = CreateFrame('Button', '$parentClose', binder, 'UIPanelCloseButton')
	close:SetPoint('TOPRIGHT', 6, 5)
	close:SetScript('OnClick', function()
		binder:Hide()
		bar = nil
	end)
	
	local function NewOverlay()
		local overlay = binder:CreateFontString(nil, nil, 'GameFontGreenLarge')
		overlay.bg = binder:CreateTexture('BACKGROUND')
		overlay.bg:SetTexture(0, 0, 0, .5)
		tinsert(overlays, overlay)
		return overlay
	end
	
	local function ResetOverlays()
		for i, overlay in ipairs(overlays) do
			overlay.bg:Hide()
			overlay:Hide()
		end
	end
	
	local function ShowOverlays()
		for i = 1, parent:GetNumChildren() do
			local button = select(i, parent:GetChildren())
			if button:IsShown() then
				local overlay = overlays[i] or NewOverlay()
				overlay:SetText(GetBindingKey('CLICK '..button:GetName()..':LeftButton'))
				overlay:SetAllPoints(button)
				overlay:Show()
				
				overlay.bg:SetAllPoints(button)
				overlay.bg:Show()
			end
		end
	end

	binder:SetScript('OnUpdate', function()
		binder:EnableKeyboard(nil)
	
		for i = 1, parent:GetNumChildren() do
			local button = select(i, parent:GetChildren())
			
			if MouseIsOver(button) and not button:GetAttribute('statehidden') then
				key = 'CLICK '..button:GetName()..':LeftButton'
				binder:EnableKeyboard(true)
			end
		end
	end)
	
	binder:SetScript('OnKeyDown', function(binder, value)
		local old = GetBindingKey(key)
		if old then
			SetBinding(old, nil)
		end
		
		if value ~= 'ESCAPE' then
			KeyBindingFrame_LoadUI()
			KeyBindingFrame.selected = key
			KeyBindingFrame.buttonPressed = KeyBindingFrameBinding1Key1Button
			KeyBindingFrame_OnKeyDown(nil, value)
			KeyBindingFrame.selected = nil
		end
		
		SaveBindings(GetCurrentBindingSet())
		binder:Update()
	end)
	
	binder.OnClick = function(new)
		bar = bar ~= new and new
		binder.Update()
	end
	
	binder.Update = function()
		ResetOverlays()
	
		if bar and bar.unlocked then
			parent = bar.parent or bar.frame
			ShowOverlays()
			
			binder:SetPoint('BOTTOM', bar.frame, 'TOP', -4, 25)
			binder:Show()
		else
			binder:Hide()
		end
	end
	
	binder:SetSize(220, text:GetHeight() + 28)
	binder:SetFrameStrata('DIALOG')
	binder:EnableMouse(true)
	binder:Hide()
	
	binder.icon = 'Interface\\Addons\\Cornucopia\\Art\\Hotkeys'
	binder.label = KEY_BINDINGS_MAC
	binder[1] = binder
	return binder
end