--[[
Copyright 2010, 2011 João Cardoso
Cornucopia is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Cornucopia.

Cornucopia is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cornucopia is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cornucopia. If not, see <http://www.gnu.org/licenses/>.
--]]

local class = select(2, UnitClass('player'))
if class == 'MAGE' then
	return
end

local Stances = Cornucopia:CreateBar('Stances', {
	sets = 'Cornucopia_Stances_Sets',
	name = 'Special Actions',
	defaults = {
		point = 'BottomLeft',
		relPoint = 'Bottom',
		x = -474, y = 62,
	}
})

function Stances:OnInitialize()
	UIPARENT_MANAGED_FRAME_POSITIONS['MultiCastActionBarFrame'] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS['ShapeshiftBarFrame'] = nil
	UIPARENT_MANAGED_FRAME_POSITIONS['PossessBarFrame'] = nil

	if class ~= 'SHAMAN' then
		-- Stances
		ShapeshiftBarFrame:UnregisterEvent('ACTIONBAR_PAGE_CHANGED')
		ShapeshiftBarFrame:DisableDrawLayer('BACKGROUND')
		ShapeshiftBarFrame:DisableDrawLayer('BORDER')
		ShapeshiftBarFrame:SetParent(self.frame)
		
		for i = 1, NUM_SHAPESHIFT_SLOTS do
			local button = _G['ShapeshiftButton'..i]
			button:ClearAllPoints()
			button:SetPoint('LEFT', self.frame, (i-1) * 37 + 3, 0)
		end
		
		function ShapeshiftBar_Update()
			if not InCombatLockdown() then
				self.frame:SetSize(max(37 * GetNumShapeshiftForms(), 1), 37)
			end
		end
	else
		-- Totems
		MultiCastActionBarFrame:SetParent(self.frame)
		MultiCastActionBarFrame:SetPoint('BOTTOMLEFT')
		MultiCastActionBarFrame.SetPoint = function() end
	
		local Update = MultiCastActionBarFrame_Update
		function MultiCastActionBarFrame_Update(multi)
			if not InCombatLockdown() then
				Update(multi)
				self.frame:SetSize(max(37 * multi.numActiveSlots, 1), 37)
			end
		end
	end
	
	-- Possess
	PossessBarFrame:SetParent(self.frame)
	PossessBarFrame:UnregisterEvent('ACTIONBAR_PAGE_CHANGED')
	PossessBarFrame:DisableDrawLayer('BACKGROUND')
	PossessBarFrame:DisableDrawLayer('BORDER')
	PossessBarFrame:SetPoint('BOTTOMLEFT')
end

function Stances:OnToolsShown()
	self.tools = CornucopiaActionsBinder or CornucopiaActions_StartBinder()
	self.OnLock, self.OnUnlock = self.tools.Update, self.tools.Update
	self.OnDoubleClick = self.tools.OnClick
	self.OnToolsShown = nil
	
	if class ~= 'SHAMAN' then
		self.parent = ShapeshiftBarFrame
	else
		self.parent = MultiCastActionBarFrame
	end
end