--[[
Copyright 2010, 2011 João Cardoso
Cornucopia is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Cornucopia.

Cornucopia is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cornucopia is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cornucopia. If not, see <http://www.gnu.org/licenses/>.
--]]

local Debuffs = Cornucopia.Bars['Debuffs'].data
local Buffs = Cornucopia:CreateBar('Buffs', {
	type = 'Movable Bar',
	sets = 'Cornucopia_Buffs_Sets',
	actualDisplay = 'BUFF_ACTUAL_DISPLAY',
	maxDisplay = 'BUFF_MAX_DISPLAY',
	buttonsName = 'BuffButton',
	filter = 'HELPFUL',
	name = 'Buffs',
	defaults = {
		point = 'TopRight',
		columns = 8, rows = 4,
		x = -174, y = -6,
		spacing = 5,
		start = ':',
	}
})


--[[ Startup ]]--

function Buffs:OnInitialize()
	for k,v in pairs(Debuffs) do self[k] = self[k] or v end
	self.UpdatePositions = BuffFrame_UpdatePositions
	self:OnSettingsChanged()
	
	function BuffFrame_UpdateAllBuffAnchors()
		local index = BuffFrame.numConsolidated > 0 and 1 or 0
		self:PlaceButton(TempEnchant1, index + 1)
		self:PlaceButton(TempEnchant2, index + 2)
		self:PlaceButton(TempEnchant3, index + 3)
		self:UpdateSize()
		
		index = index + BuffFrame.numEnchants
		for i = 1, min(BUFF_ACTUAL_DISPLAY, BUFF_MAX_DISPLAY - index) do
			local buff = _G['BuffButton'..i]
			if buff then
				if buff.consolidated then
					buff:SetParent(ConsolidatedBuffsContainer)
				else
					tinsert(self.buttons, buff)
				end
			end
		end
		self:PlaceButtons(index)
	end

	function BuffFrame_UpdatePositions()
		self:PlaceButton(ConsolidatedBuffs, 1)
		self.UpdatePositions()
	end

	TempEnchant1:Hide()
	TempEnchant2:Hide()
	TempEnchant3:Hide()
end


--[[ Config ]]--

function Buffs:OnUnlock()
	if not self.GetEnchants then
		self.GetEnchants = GetWeaponEnchantInfo
		
		function GetWeaponEnchantInfo(...)
			if self.locked then
				return self.GetEnchants(...)
			end
		end
	end
	
	Debuffs.OnUnlock(self)
end

function Buffs:OnSettingsChanged()
	self:PlaceButton(ConsolidatedBuffs, 1)
	Debuffs.OnSettingsChanged(self)
end