--[[
Copyright 2010, 2011 João Cardoso
Cornucopia is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Cornucopia.

Cornucopia is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cornucopia is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cornucopia. If not, see <http://www.gnu.org/licenses/>.
--]]

local Toolbar = Cornucopia:CreatePanel('Toolbar', 'TOP')
local Dropdown = CreateFrame('Frame', '&parentDropdown', Toolbar, 'UIDropDownMenuTemplate')
local L, Buttons  = Cornucopia_Locals, {}


--[[ Startup ]]--

function Toolbar:Startup()	
	-- Close Button
	CornucopiaToolbarCloseButton:SetPoint('TOPRIGHT', 4, 1)
	CornucopiaToolbarCloseButton:SetScript('OnClick', function()
		Cornucopia:HideConfig()
	end)
	
	local corner = self:CreateTexture()
	corner:SetTexture('Interface\\DialogFrame\\UI-DialogBox-Corner')
	corner:SetPoint('TOPRIGHT', 0, -3)
	corner:SetSize(31, 31)

	-- Border
	CornucopiaToolbarTopTileStreaks:SetPoint('TOPRIGHT', -2, -6)
	CornucopiaToolbarTopTileStreaks:SetPoint('TOPLEFT', 2, -6)
	CornucopiaToolbarBg:SetPoint('TOPLEFT', 2, -6)
	CornucopiaToolbarTitleBg:Hide()
	
	CornucopiaToolbarTopBorder:SetTexCoord(0, 1, 0.2734375, 0.203125)
	CornucopiaToolbarTopBorder:SetPoint('TOPRIGHT', -11, 1)
	CornucopiaToolbarTopBorder:SetPoint('TOPLEFT', 8, 1)
	CornucopiaToolbarTopBorder:SetHeight(9)
	
	CornucopiaToolbarRightBorder:SetPoint('TOPRIGHT', 1, -10)
	CornucopiaToolbarTopRightCorner:SetTexCoord(0.1328125, 0.21875, 0.984375, 0.8984375)
	CornucopiaToolbarTopRightCorner:SetPoint('TOPRIGHT', 0, 1)
	CornucopiaToolbarTopRightCorner:SetSize(11, 11)
	
	CornucopiaToolbarTopLeftCorner:SetTexCoord(0.0078125, 0.1171875, 0.7421875, 0.6328125)
	CornucopiaToolbarTopLeftCorner:SetSize(14, 14)
	
	-- Buttons
	self:ShowBar(nil)
end

function Toolbar:ShowBar(bar)
	self:ClearButtons()
	self:AddDefault('Add')
	self:AddDefault('Options')
	self:AddDefault('Help')
		
	if not IsControlKeyDown() and bar then
		local data = bar.data
		if data.OnToolsShown then
			data:OnToolsShown(self)
		end
		
		local buttons = data.tools
		if buttons or _Ungroup_ then
			self.size = self.size + 39
			
			for i, button in ipairs(buttons) do
				self:AddButton(button.label or button.name or button.text, button.icon, button.OnClick)
			end
			
			if _Ungroup_ then
				self:AddDefault('Ungroup')
			end
		end
		
		self.data = data
	end
	
	self:SetSize(self.size + 15, 53)
end


--[[ Buttons ]]--

function Toolbar:AddDefault(name)
	self:AddButton(L[name] or name, 'Interface\\Addons\\Cornucopia\\Art\\'..name, self['Show'..name])
end

function Toolbar:AddButton(name, icon, method)
	local button = Buttons[self.index] or self:CreateButton()
	button:SetPoint('TOPLEFT', self.size, -8)
	button:SetNormalTexture(icon)
	button.text:SetText(name)
	button.method = method
	button:Show()
	
	self.size = self.size + max(39, button.text:GetStringWidth() + 4)
	self.index = self.index + 1
end

function Toolbar:CreateButton()
	local button = CreateFrame('Button', nil, self)
	button:SetScript('OnDragStart', function() self:StartMoving() end)
	button:RegisterForDrag('LeftButton')
	button:SetSize(29, 29)
	
	local text = button:CreateFontString()
	text:SetFontObject('GameFontHighlightSmall')
	text:SetPoint('TOP', button, 'BOTTOM')
	button.text = text
	
	button:SetScript('OnMouseDown', function(_, key)
		button:GetNormalTexture():SetVertexColor(.7,.7,.7)
		text:SetTextColor(.7,.7,.7)
		
		if button.method then
			button.method(self.data, button, key)
		end
	end)
	
	button:SetScript('OnMouseUp', function()
		button:GetNormalTexture():SetVertexColor(1,1,1)
		text:SetTextColor(1,1,1)
	end)
	
	button:SetScript('OnDragStop', function()
		button:GetScript('OnMouseUp')()
		self:StopMovingOrSizing()
	end)
	
	tinsert(Buttons, button)
	return button
end

function Toolbar:ClearButtons()
	for i, button in pairs(Buttons) do
		button:Hide()
	end
	self.index = 1
	self.size = 13
end


--[[ OnClick ]]--

local bars = {}
local function AddLine(data, method)
	tinsert(bars, {
		text = (data.icon and ('|T'..data.icon..':0|t ') or '') .. (data.name or id),
		notCheckable = 1,
   		func = method
	})
end

local function ShowBar(id)
	local bar = Cornucopia:GetBar(id)
	bar.sets.hide = nil
	bar:UpdatePosition()
	bar.config:Show()
	bar:OnMouseDown()
	bar:Show()
	
	if bar.data.OnAdd then
		bar.data.OnAdd()
	end
end

function Toolbar:ShowAdd(button)
	wipe(bars)
	
	for id, data in Cornucopia:IterateGroups() do
		if data.OnAddAttempt then
			data:OnAddAttempt()
		end
	
		if not data.cannotAdd and (not data.limit or data.limit < #data.sets) then
			AddLine(data, function()
    			local sets = CopyTable(data.barDefaults or {})
    			tinsert(data.sets, sets)
    			
    			local id = id..#data.sets
    			local new = setmetatable({}, data)
				new.index = #data.sets
				new.sets = sets
				
				Cornucopia:CreateBar(id, new)
				Cornucopia:InitializeBar(new.frame)
				Cornucopia.InitializeConfig(new.frame)
				ShowBar(id)
    		end)
		end
	end
	
	for id, bar in Cornucopia:IterateBars() do
		if bar.sets.hide then
			AddLine(bar.data, function()
				ShowBar(id)
    		end)
		end
	end
	
	if #bars == 0 then
		bars[1] = {
			text = L['There are no bars to be added'],
			disabled = true,
			notCheckable = 1,
		}
	end
	
	EasyMenu(bars, Dropdown, button, 0, -13, 'MENU')
end

local options = {
	{
	    text = ENABLE,
	    notCheckable = 1,
	    isTitle = 1
   	},
	{
	    text = L['Rulers'],
	    tooltipTitle = L['Rulers'],
	    tooltipText = L['You can hold "Alt" to temporarily|ndisable/enable the rulers.'],
	    func = function() Cornucopia_HideRulers = not Cornucopia_HideRulers or nil end,
	    checked = function() return not Cornucopia_HideRulers end,
	    tooltipOnButton = 1,
	    isNotRadio = 1
   	},
   	{
	    text = L['Minimap Button'],
	    tooltipTitle = L['Minimap Button'],
	    tooltipText = L['If you do not want to use the minimap|nbutton to enter configuration mode,|nyou can set a key binding for it.'],
	    func = function()
	    	if Cornucopia_HideButton then
	    		CornucopiaButton:Show()
	    		Cornucopia_HideButton = nil
	    	else
	    		CornucopiaButton:Hide()
	    		Cornucopia_HideButton = true
	    	end
	    end,
	    
	    checked = function() return not Cornucopia_HideButton end,
	    tooltipOnButton = 1,
	    isNotRadio = 1
   	},
   	{
	    text = L['Background'],
	    notCheckable = 1,
		isTitle = 1
   	},
	{
	    text = L['Translucent'],
	    checked = function() return not Cornucopia_Opaque end,
	    func = function()
	    	Cornucopia_Opaque = nil
	    	Cornucopia:UpdateBackdrop()
	    end,
   	},
   	{
	    text = L['Opaque'],
		checked = function() return Cornucopia_Opaque end,
	    func = function()
	    	Cornucopia_Opaque = true
	    	Cornucopia:UpdateBackdrop()
	    end,
   	},
}

function Toolbar:ShowOptions(button)
	EasyMenu(options, Dropdown, button, 0, -13, 'MENU')
end

function Toolbar:ShowHelp()
	Cornucopia:TriggerTutorial(1, true)
end

Toolbar:Startup()